Package: RaMS
Type: Package
Title: R Access to Mass-Spec Data
Version: 1.4.3
Authors@R: c(
  person(given = "William", family = "Kumler", email = "wkumler@uw.edu",
         role = c("aut", "cre", "cph")),
  person(given = "Ricardo", family = "Cunha", role = "ctb"),
  person(given = "Ethan", family = "Bass", role = "ctb")
  )
Maintainer: William Kumler <wkumler@uw.edu>
Description: R-based access to mass-spectrometry (MS) data. While many packages 
  exist to process MS data, many of these make it difficult to 
  access the underlying mass-to-charge ratio (m/z), intensity, and 
  retention time of the files 
  themselves. This package is designed to format MS data in a tidy fashion and 
  allows the user perform the plotting and analysis.
License: MIT + file LICENSE
URL: https://github.com/wkumler/RaMS
BugReports: https://github.com/wkumler/RaMS/issues
Encoding: UTF-8
Imports: xml2, base64enc, data.table, utils, stats
RoxygenNote: 7.3.1
Suggests: testthat, knitr, rmarkdown, tidyverse, ggplot2, dplyr,
        plotly, openxlsx, DBI, RSQLite, reticulate, BiocParallel,
        Spectra, arrow, microbenchmark, rvest
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2024-10-09 00:15:13 UTC; willi
Author: William Kumler [aut, cre, cph],
  Ricardo Cunha [ctb],
  Ethan Bass [ctb]
Repository: CRAN
Date/Publication: 2024-10-09 05:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 14:32:52 UTC; unix
