# Generated by roxygen2: do not edit by hand

S3method("$",msdata_connection)
S3method("[",msdata_connection)
S3method(print,msdata_connection)
export("%between%")
export(between)
export(editMSfileRTs)
export(grabAccessionData)
export(grabMSdata)
export(grabMzmlData)
export(grabMzxmlData)
export(minifyMSdata)
export(msdata_connection)
export(mz_group)
export(pmppm)
export(qplotMS1data)
export(tmzmlMaker)
export(trapz)
import(data.table)
import(utils)
import(xml2)
importFrom(base64enc,base64decode)
importFrom(base64enc,base64encode)
importFrom(data.table,"%between%")
importFrom(stats,approx)
